@@ -1,14 +1,15 @@
+import abc
 import dataclasses
 import logging
 import urllib.parse
 from types import UnionType
 from typing import Any, Optional, Dict, Union
 
-from aiohttp import Payload
-from aiohttp.web_response import Response
+import aiohttp.web
+import aiohttp.abc
 from dynamic_types.class_name import _prepare_class_name
 from dynamic_types.create import create_type, create_dataclass
-from init_helpers import custom_dumps, is_instance
+from init_helpers import custom_dumps, is_instance, raise_if
 from init_helpers.dict_to_dataclass import dict_to_dataclass, convert_to_type
 from yarl import URL
 
@@ -33,7 +34,9 @@ _FIXED_PAYLOAD_TYPE__NAME = "_fixed_payload_type"
 _FIXED_ERROR_CODE__NAME = "_fixed_error_code_type"
 
 
-class Answer(Response):
+class BaseAnswer(abc.ABC):
+    # noinspection PyUnusedLocal
+    @abc.abstractmethod
     def __init__(self,
                  payload: Union[str, bytes, None],
                  status: HttpStatusCode = None,
@@ -41,17 +44,7 @@ class Answer(Response):
                  headers: Optional[Dict[str, str]] = None,
                  charset: Charset = None,
                  ):
-        if status is None and hasattr(self, _FIXED_STATUS_CODE__NAME):
-            status = getattr(self, _FIXED_STATUS_CODE__NAME)
-
-        status_code = status or self.get_class_status_code() or 200
-        content_type = content_type or self.get_class_content_type()
-        content_type = content_type.value if isinstance(content_type, ContentType) else content_type
-        super().__init__(body=payload, status=status_code, content_type=content_type, headers=headers, charset=charset)
-
-    @property
-    def bytes_body(self) -> bytes:
-        return self.body._value if isinstance(self.body, Payload) else self.body
+        raise NotImplementedError
 
     def __class_getitem__(cls, modifier: Union[HttpStatusCode, ContentType]) -> type:
         field_name_to_value = {}
@@ -67,7 +60,6 @@ class Answer(Response):
             raise TypeError(f"Unexpected modifier type: {type(modifier)}")
 
         new_type = create_type(_prepare_class_name([cls], modifier), [cls], field_name_to_value)
-        # new_type = type(_prepare_class_name([cls], modifier), (cls, ), field_name_to_value)
         return new_type
 
     @classmethod
@@ -85,6 +77,81 @@ class Answer(Response):
         return bytes
 
 
+class StreamAnswer(BaseAnswer, aiohttp.web.StreamResponse):
+    def __init__(self,
+                 payload: str | bytes | None,
+                 status: HttpStatusCode = None,
+                 content_type: ContentType | str | None = None,
+                 headers: dict[str, str] | None = None,
+                 charset: Charset = None,
+                 ):
+        if status is None and hasattr(self, _FIXED_STATUS_CODE__NAME):
+            status = getattr(self, _FIXED_STATUS_CODE__NAME)
+        headers = headers or {}
+
+        status_code = status or self.get_class_status_code() or 200
+        content_type = content_type or self.get_class_content_type()
+        content_type = content_type.value if isinstance(content_type, ContentType) else content_type
+
+        if isinstance(payload, str):
+            content_type = content_type or "text/plain"
+            charset = charset or "utf-8"
+            payload = payload.encode(charset)
+
+        raise_if(
+            payload is not None and not isinstance(payload, bytes),
+            TypeError(f'Payload must be a str | bytes | None, got {type(payload)}'))
+
+        if content_type is not None:
+            content_type_header_value = content_type + ('' if charset is not None else f'; charset={charset}')
+            headers[aiohttp.hdrs.CONTENT_TYPE] = content_type_header_value
+        aiohttp.web.StreamResponse.__init__(self, status=status_code, headers=headers)
+        self.initial_data: bytes | None = payload
+
+    async def prepare(self, request: aiohttp.web.BaseRequest) -> Optional[aiohttp.abc.AbstractStreamWriter]:
+        result = await super().prepare(request)
+        await self._write_initial_data()
+        return result
+
+    async def _write_initial_data(self) -> None:
+        if self.initial_data is not None:
+            initial_data, self.initial_data = self.initial_data, None
+            await aiohttp.web.StreamResponse.write(self, initial_data)
+
+    async def write(self, data: bytes) -> None:
+        await self._write_initial_data()
+        await aiohttp.web.StreamResponse.write(self, data)
+
+    def __class_getitem__(cls, modifier: Union[HttpStatusCode, ContentType, type, UnionType]) -> type:
+        return super().__class_getitem__(modifier)
+
+
+class Answer(BaseAnswer, aiohttp.web.Response):
+    def __init__(self,
+                 payload: Union[str, bytes, None],
+                 status: HttpStatusCode = None,
+                 content_type: Optional[Union[ContentType, str]] = None,
+                 headers: Optional[Dict[str, str]] = None,
+                 charset: Charset = None,
+                 ):
+        if status is None and hasattr(self, _FIXED_STATUS_CODE__NAME):
+            status = getattr(self, _FIXED_STATUS_CODE__NAME)
+
+        status_code = status or self.get_class_status_code() or 200
+        content_type = content_type or self.get_class_content_type()
+        content_type = content_type.value if isinstance(content_type, ContentType) else content_type
+        aiohttp.web.Response.__init__(
+            self, body=payload, status=status_code, content_type=content_type, headers=headers, charset=charset)
+
+    @property
+    def bytes_body(self) -> bytes:
+        # noinspection PyProtectedMember
+        return self.body._value if isinstance(self.body, aiohttp.Payload) else self.body
+
+    def __class_getitem__(cls, modifier: Union[HttpStatusCode, ContentType, type, UnionType]) -> type:
+        return super().__class_getitem__(modifier)
+
+
 class JsonableAnswer(Answer):
     """
     Class accepts dict, list, set, str, int, float, bool, None, dataclass instances
@@ -146,7 +213,7 @@ class WrappedAnswer(JsonableAnswer):
 
     @classmethod
     def _wrap_type(cls, type_: type) -> type:
-        attribute_key_to_type = {"done": bool, "result": type_} if type_ not  in (None, type[None]) else {"done": bool}
+        attribute_key_to_type = {"done": bool, "result": type_} if type_ not in (None, type[None]) else {"done": bool}
         return create_dataclass(_prepare_class_name([type_],  "Wrapped"), [cls], attribute_key_to_type)
 
     def __class_getitem__(cls, modifier: Union[HttpStatusCode, ContentType, type, UnionType, type[None]]) -> type:
@@ -180,11 +247,9 @@ class ErrorAnswer(JsonableAnswer):
                  headers: Optional[Dict[str, str]] = None,
                  charset: Charset = Charset.UTF8) -> None:
         error_code = error_code or self.get_class_error_code()
-        super().__init__(payload=ErrorDescription(error, error_type, error_code),
-                         status=status,
-                         headers=headers,
-                         charset=charset,
-                         )
+        super().__init__(
+            payload=ErrorDescription(error, error_type, error_code),
+            status=status, headers=headers, charset=charset)
 
     def __class_getitem__(cls, modifier: Union[HttpStatusCode, ContentType, ErrorCode, type, None]) -> type:
         if isinstance(modifier, ErrorCode):
@@ -210,20 +275,16 @@ class ExceptionAnswer(ErrorAnswer):
                  error_code: Optional[ErrorCode] = None,
                  headers: Optional[Dict[str, str]] = None,
                  charset: Charset = Charset.UTF8) -> None:
-        super().__init__(error=str(exception),
-                         status=status,
-                         error_type=type(exception).__name__,
-                         error_code=error_code,
-                         headers=headers,
-                         charset=charset,
-                         )
+        super().__init__(
+            error=str(exception), status=status, error_type=type(exception).__name__, error_code=error_code,
+            headers=headers, charset=charset)
 
 
 class RedirectAnswer(Answer):
     def __init__(self, location: str, headers: Optional[Dict[str, str]] = None):
         headers = {} if headers is None else headers
         headers['Location'] = str(URL(location))
-        super().__init__(None, HttpStatusCode.Found, headers=headers)
+        Answer.__init__(self, None, HttpStatusCode.Found, headers=headers)
 
     @property
     def location(self):
